//////////////////////////////////////////////////////////////////////////////
//  Copyright (c) Compaq Computer Corporation, 1999                         //
//                                                                          //
//  All Rights Reserved.  Unpublished rights reserved under the  copyright  //
//  laws of the United States.                                              //
//                                                                          //
//  The software contained on this media is proprietary  to  and  embodies  //
//  the   confidential   technology   of  Compaq Computer  Corporation.     //
//  Possession, use, duplication or  dissemination  of  the  software  and  //
//  media  is  authorized  only  pursuant  to a valid written license from  //
//  Compaq Computer Corporation.                                            //
//////////////////////////////////////////////////////////////////////////////

import robotcontroller.*;
import java.lang.*;

/**
 * Class main method main is run from the .profile file.  It should configure the robot and run.
 * This is a nonsensical test robot to illustrate the use of Behavior, Alarm, etc.
 */
public class main {
    /**
     * activeBehavior is static so that it is accessible in nested classes -- the behaviors -- and can be modified.
     */
    static Behavior activeBehavior = null;	
    static boolean useAlarmThreads = false;
    public static void main(String args[]) {
	if (args.length > 0) {
	    useAlarmThreads = true;
	}
        try {
            /* these are all final so that they can be accessed from nested classes.  
             * Another option is to make them static, as activeBehavior is (because it's not constant, and so cannot be final).
             */
            final Controller controller = Controller.getController();
            /* all the buttons */
            final AnalogPort ap27 = new AnalogPort(AnalogPort.AIN27_PORT);;
            final AnalogPort ap28 = new AnalogPort(AnalogPort.AIN28_PORT);
            final AnalogPort ap29 = new AnalogPort(AnalogPort.AIN29_PORT);
            final AnalogPort ap30 = new AnalogPort(AnalogPort.AIN30_PORT);
            final Motor motors[] = new Motor[8];
            final LCD lcd = new LCD();

            /**
             * Configure as many bidirectional motors as we are allowed.
             */
	    motors[0] = new Motor(0,4);
	    motors[1] = new Motor(5,9);
	    motors[2] = new Motor(10,14);
	    motors[3] = new Motor(1,2);
	    motors[4] = new Motor(6,7);
	    motors[5] = new Motor(11,12);
	    motors[6] = new Motor(16,17);
	    motors[7] = new Motor(8,18);

            lcd.clear();
            lcd.setCursor(0, 0);
            lcd.writeString("rev up...");
            lcd.setCursor(1, 0);
            lcd.writeString("  with RoboJava!!!");

            /* 
             * now for some behaviors -- these are anonymous inner (nested)
             * classes, which can refer to statics, things they allocate,
             * and final variables in the enclosing context (lexical
             * scope).  
             * 
             * Behaviors are initially disabled.  When enabled, the behave
             * method is called once per cycle from controller.run or from the
             * Controller thread (if controller.startControllerThread is invoked.
             */

            /*
             * stop sets the speed of all motors to 0
             */
            final Behavior stop = new Behavior("stopMotors") {
                public void behave () { 
                    lcd.clear();
                    for (int m = 0; m < 8; m++) motors[m].setSpeed(Motor.STOPPED);
                };
            };

            /*
             * forward sets the speed of all motors to max forward
             */
            final Behavior forward = new Behavior("goForward") {
                public void behave () { 
                    lcd.clear();
                    lcd.setCursor(1, 0);
                    lcd.writeString("  forward");
                    for (int m = 0; m < 8; m++) motors[m].setSpeed(Motor.MAXFORWARD);
                }
            };

            /*
             * reverse sets the speed of all motors to max reverse
             */
            final Behavior reverse = new Behavior("goBackwards") {
                public void behave () { 
                    lcd.clear();
                    lcd.setCursor(1, 0);
                    lcd.writeString("  reverse");
                    for (int m = 0; m < 8; m++) motors[m].setSpeed(Motor.MAXREVERSE);
                }
            };

            /*
             * The enabler behavior keeps track of the currently active
             * behavior and switches among stop, forward, and reverse
             * depending on button pushes on three of the buttons.  
             */
            activeBehavior = stop;            
            final Behavior enabler = new Behavior("enabler") {
                public void behave() {
                    if (ap27.getValue() > 128) {
                        activeBehavior.disable();
                        activeBehavior = stop;
                        activeBehavior.enable();
                    }
                    if (ap28.getValue() > 128) {
                        activeBehavior.disable();
                        activeBehavior = forward;
                        activeBehavior.enable();
                    }
                    if (ap29.getValue() > 128) {
                        activeBehavior.disable();
                        activeBehavior = reverse;
                        activeBehavior.enable();
                    }
                }
            };

	    if (useAlarmThreads) {
		Alarm.useThreadsForAlarms = true;
	    }
            /* we wait ten seconds, then enable the enabler -- before that, nothing happens */
            Alarm tenSeconds = new Alarm(10000, "ten second alarm") {
                public void timesup() {
                    System.err.println("currentTimeMillis=" + Long.toString(System.currentTimeMillis()));
                    System.err.println("10 seconds have elapsed.");
                    lcd.clear();
                    lcd.setCursor(0,0);
                    lcd.writeString("10 seconds.");
                    enabler.enable();
                };
            };

            /* 
             * Contest rules require us to stop after 60 seconds.
             */
            Alarm timesup = new Alarm(60000, "time's up alarm") {
                public void timesup() throws ControllerException {
		    synchronized (Controller.getController()) {
			System.err.println("currentTimeMillis=" + Long.toString(System.currentTimeMillis()));
			System.err.println("60 seconds have elapsed.  Exiting");
			lcd.clear();
			lcd.setCursor(0,0);
			lcd.writeString("60 seconds.");
			lcd.setCursor(1,0);
			lcd.writeString("  stopping motors...");

			for (int m = 0; m < 8; m++) motors[m].setSpeed(Motor.STOPPED);

			/* we use the override method to force a stop */
			System.exit(0);
		    }
                };
            };

            /* get more debug messages from run */
            controller.setVerbose(false);
            /* change cycle time to .1 second */
            controller.cycleTime = 100; /* milliseconds */
	    
	    Motor.nitroEnable();
            /* make it go */
            controller.run();

	} catch (Exception e) {
            System.out.println("APMISNUM Exception: "+e); 
            e.printStackTrace();
        }

    }	
};
