/****************************************************************************/
/*  Copyright (c) Compaq Computer Corporation, 1999                         */
/*                                                                          */
/*  All Rights Reserved.  Unpublished rights reserved under the  copyright  */
/*  laws of the United States.                                              */
/*                                                                          */
/*  The software contained on this media is proprietary  to  and  embodies  */
/*  the   confidential   technology   of  Compaq Computer  Corporation.     */
/*  Possession, use, duplication or  dissemination  of  the  software  and  */
/*  media  is  authorized  only  pursuant  to a valid written license from  */
/*  Compaq Computer Corporation.                                            */
/****************************************************************************/


package robotcontroller;

/**
 * Represents one skfrdc servo port.  Setting the pulse width of a servo controls the servo position.
 */
public class Servo {
    static Controller rdc;
    int port;
    static int portsInUse;

   /**
    * timer period that will cause a servo to move all the way to the left.
    */
   public static final int MAX_LEFT = 60928;
   /**
    * timer period that will cause a servo to move to the middle.
    */
   public static final int MIDDLE = 63036;
   /**
    * timer period that will cause a servo to move all the way to the right.
    */
   public static final int MAX_RIGHT = 65056;
    
    /**
     * Creates object a new servo on the specified port.
     *
     * @param port the port to use, ranges from 0 to 3.
     * @exception ControllerException if the port is out of range or already in use.
     */
    public Servo(int port) throws ControllerException
    {
        if (rdc == null)
            rdc = Controller.getController();
        synchronized (rdc) {
            int portMask = (1 << port);
            this.rdc = rdc;
            this.port = port;
            if ((portMask & portsInUse) != 0)
                throw new ControllerException("servo port already in use");
            portsInUse |= portMask;
        }
    };
    /**
     * Releases a port for reuse.
     */
    public void finalize()
    {
        synchronized (rdc) {
            portsInUse &= ~(1 << port);
        }
    };

    public void enable() throws ControllerError
    {
        synchronized (rdc) {
            rdc.servo_enable_port(port, 1);
        }
    };
    public void disable() throws ControllerError
    {
        synchronized (rdc) {
            rdc.servo_enable_port(port, 0);
        }
    };

    /**
     * Setting the pulse width (timer period) controls the servo position.
     */
    public void setPulseWidth(int pw) throws ControllerError
    {
        synchronized (rdc) {
            rdc.servo_set_port(port, pw);
        }
    };
};
