/****************************************************************************/
/*  Copyright (c) Compaq Computer Corporation, 1999                         */
/*                                                                          */
/*  All Rights Reserved.  Unpublished rights reserved under the  copyright  */
/*  laws of the United States.                                              */
/*                                                                          */
/*  The software contained on this media is proprietary  to  and  embodies  */
/*  the   confidential   technology   of  Compaq Computer  Corporation.     */
/*  Possession, use, duplication or  dissemination  of  the  software  and  */
/*  media  is  authorized  only  pursuant  to a valid written license from  */
/*  Compaq Computer Corporation.                                            */
/****************************************************************************/

#include <jni.h>
#include "robotcontroller_Controller.h"
#include "rdcapi.h"


void throwControllerException(JNIEnv *jnienv, const char *msg)
{
   jclass rseclass = (*jnienv)->FindClass(jnienv, "robotcontroller.ControllerException");
   if ((*jnienv)->ExceptionOccurred(jnienv))
       return;
   (*jnienv)->ThrowNew(jnienv, rseclass, msg);
}

void throwControllerError(JNIEnv *jnienv, const char *msg)
{
   jclass rseclass = (*jnienv)->FindClass(jnienv, "robotcontroller.ControllerError");
   if ((*jnienv)->ExceptionOccurred(jnienv))
       return;
   (*jnienv)->ThrowNew(jnienv, rseclass, msg);
}


/*
 * Class:     Controller_Controller
 * Method:    usb_init
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_usb_1init
  (JNIEnv *jnienv, jobject jcontroller)
{
   int rc;
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorEnter(jnienv, jcontroller);
   if (rc < 0) {
       throwControllerError(jnienv, "failed to enter monitor");
       return;
   }
#endif
   rc = rdc_usb_init();
   if (rc < 0)
      throwControllerError(jnienv, "rdc_usb_init failed");
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorExit(jnienv, jcontroller);   
   if (rc < 0) {
       throwControllerError(jnienv, "failed to exit monitor");
   }
#endif
}

/*
 * Class:     Controller_Controller
 * Method:    aram_read
 * Signature: (II[B)V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_aram_1read
    (JNIEnv *jnienv, jobject jcontroller, jint addr, jint len, jbyteArray jdata)
{
    jsize alen = (*jnienv)->GetArrayLength(jnienv, jdata);
    if ((*jnienv)->ExceptionOccurred(jnienv))
        return;
    if (alen >= len) {
        jbyte *data = (*jnienv)->GetByteArrayElements(jnienv, jdata, NULL);
        int rc;
#ifdef ROBOSKIFF_MONITORS
        rc = (*jnienv)->MonitorEnter(jnienv, jcontroller);
        if (rc < 0) {
            throwControllerError(jnienv, "failed to enter monitor");
            return;
        }
#endif
        rc = rdc_aram_read(addr, len, data);
#ifdef ROBOSKIFF_MONITORS
        (*jnienv)->MonitorExit(jnienv, jcontroller);   
#endif
        
        (*jnienv)->ReleaseByteArrayElements(jnienv, jdata, data, JNI_ABORT);
        if (rc < 0)
            throwControllerError(jnienv, "rdc_aram_read failed");
    } else {
        throwControllerError(jnienv, "aram_read: array too small");
    }
}

/*
 * Class:     Controller_Controller
 * Method:    aram_write
 * Signature: (II[B)V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_aram_1write
(JNIEnv *jnienv, jobject jcontroller, jint addr, jint len, jbyteArray jdata)
{
   jsize alen = (*jnienv)->GetArrayLength(jnienv, jdata);
   if ((*jnienv)->ExceptionOccurred(jnienv))
       return;
   if (alen >= len) {
      jbyte *data = (*jnienv)->GetByteArrayElements(jnienv, jdata, NULL);
      int rc = rdc_aram_write(addr, len, data);
      (*jnienv)->ReleaseByteArrayElements(jnienv, jdata, data, 0);
      if (rc < 0)
         throwControllerError(jnienv, "rdc_aram_write failed");
   } else {
      throwControllerError(jnienv, "aram_write: array too small");
   }
}

/*
 * Class:     Controller_Controller
 * Method:    eeprom_read
 * Signature: (II[B)V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_eeprom_1read
(JNIEnv *jnienv, jobject jcontroller, jint addr, jint len, jbyteArray jdata)
{
   jsize alen = (*jnienv)->GetArrayLength(jnienv, jdata);
   if ((*jnienv)->ExceptionOccurred(jnienv))
       return;
   if (alen >= len) {
      jbyte *data = (*jnienv)->GetByteArrayElements(jnienv, jdata, NULL);
      int rc = rdc_eeprom_read(addr, len, data);
      (*jnienv)->ReleaseByteArrayElements(jnienv, jdata, data, JNI_ABORT);
      if (rc < 0)
         throwControllerError(jnienv, "rdc_eeprom_read failed");
   } else {
      throwControllerError(jnienv, "eeprom_read: array too small");
   }
}

/*
 * Class:     Controller_Controller
 * Method:    eeprom_write
 * Signature: (II[B)V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_eeprom_1write
(JNIEnv *jnienv, jobject jcontroller, jint addr, jint len, jbyteArray jdata)
{
   jsize alen = (*jnienv)->GetArrayLength(jnienv, jdata);
   if ((*jnienv)->ExceptionOccurred(jnienv))
       return;
   if (alen >= len) {
      jbyte *data = (*jnienv)->GetByteArrayElements(jnienv, jdata, NULL);
      int rc = rdc_eeprom_write(addr, len, data);
      (*jnienv)->ReleaseByteArrayElements(jnienv, jdata, data, 0);
      if (rc < 0)
         throwControllerError(jnienv, "rdc_eeprom_write failed");
   } else {
      throwControllerError(jnienv, "aram_read: array too small");
   }
}

/*
 * Class:     Controller_Controller
 * Method:    get_eeprom_width
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_Controller_Controller_get_1eeprom_1width
  (JNIEnv *jnienv, jobject jcontroller)
{
   char width = 0;
   int rc = rdc_eeprom_get_width(&width);
   if (rc < 0)
      throwControllerError(jnienv, "rdc_get_eeprom_width failed");
   return width;
}

/*
 * Class:     Controller_Controller
 * Method:    get_usb_chip_rev
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_Controller_Controller_get_1usb_1chip_1rev
  (JNIEnv *jnienv, jobject jcontroller)
{
   char rev = 0;
   int rc = rdc_get_usb_chip_rev(&rev);
   if (rc < 0)
      throwControllerError(jnienv, "rdc_get_usb_chip_rev failed");
   return rev;
}

/*
 * Class:     Controller_Controller
 * Method:    usb_renumerate
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_usb_1renumerate
  (JNIEnv *jnienv, jobject jcontroller)
{
   int rc = rdc_usb_renumerate();
   if (rc < 0)
      throwControllerError(jnienv, "rdc_usb_renumerate failed");
}


/*
 * Class:     Controller_Controller
 * Method:    adc_get_values
 * Signature: ([B)I
 */
JNIEXPORT void JNICALL Java_Controller_Controller_adc_get_values
    (JNIEnv *jnienv, jobject jcontroller, jbyteArray jdata)
{
    jsize alen = (*jnienv)->GetArrayLength(jnienv, jdata);
   if ((*jnienv)->ExceptionOccurred(jnienv))
       return;
    if (alen >= 32) {
        jbyte *data;
        int rc;
#ifdef ROBOSKIFF_MONITORS
        rc = (*jnienv)->MonitorEnter(jnienv, jcontroller);
        if (rc < 0) {
            throwControllerError(jnienv, "failed to enter monitor");
            return;
        }
#endif
        data = (*jnienv)->GetByteArrayElements(jnienv, jdata, NULL);
        rc = rdc_adc_get_vals(data);
        (*jnienv)->ReleaseByteArrayElements(jnienv, jdata, data, JNI_ABORT);
#ifdef ROBOSKIFF_MONITORS
        (*jnienv)->MonitorExit(jnienv, jcontroller);   
#endif
        if (rc < 0)
            throwControllerError(jnienv, "get_adc_values failed");
    } else {
        throwControllerError(jnienv, "get_adc_values: destination array too small");
    }
}

/*
 * Class:     Controller_Controller
 * Method:    motor_set_vec
 * Signature: (I[B)V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_motor_set_vec
    (JNIEnv *jnienv, jobject jcontroller, jint reg, jbyteArray jdata)
{
    jsize alen = (*jnienv)->GetArrayLength(jnienv, jdata);
   if ((*jnienv)->ExceptionOccurred(jnienv))
       return;
    if (alen >= MTR_VECLEN) {
        jbyte *data = (*jnienv)->GetByteArrayElements(jnienv, jdata, NULL);
        int rc, rc1;
#ifdef ROBOSKIFF_MONITORS
        rc = (*jnienv)->MonitorEnter(jnienv, jcontroller);
        if (rc < 0) {
            throwControllerError(jnienv, "failed to enter monitor");
            return;
        }
#endif
        rc = rdc_motor_set_motor_vec(reg, data);
#ifdef ROBOSKIFF_MONITORS
        rc1 = (*jnienv)->MonitorExit(jnienv, jcontroller);   
        if (rc1 < 0) {
            throwControllerError(jnienv, "failed to exit monitor");
        }
#endif
        (*jnienv)->ReleaseByteArrayElements(jnienv, jdata, data, 0);
        if (rc < 0)
            throwControllerError(jnienv, "rdc_set_motor_vec failed");
    } else {
        throwControllerError(jnienv, "set_motor_vec: array too small");
    }
}

/*
 * Class:     Controller_Controller
 * Method:    servo_enable_port
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_servo_enable_port
  (JNIEnv *jnienv, jobject jcontroller, jint port, jint state)
{
   int rc;
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorEnter(jnienv, jcontroller);
   if (rc < 0) {
       throwControllerError(jnienv, "failed to enter monitor");
       return;
   }
#endif
   rc = rdc_servo_enable_port(port, state);
   if (rc < 0)
      throwControllerError(jnienv, "rdc_servo_enable_port failed");
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorExit(jnienv, jcontroller);   
   if (rc < 0) {
       throwControllerError(jnienv, "failed to exit monitor");
   }
#endif
}

/*
 * Class:     Controller_Controller
 * Method:    servo_set_port
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_servo_set_port
  (JNIEnv *jnienv, jobject jcontroller, jint port, jint period)
{
   int rc;
   if (period < SERVO_MAX_LEFT || period > SERVO_MAX_RIGHT) {
      throwControllerException(jnienv, "period out of range for servo_set_port");
      return;
   }
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorEnter(jnienv, jcontroller);
   if (rc < 0) {
       throwControllerError(jnienv, "failed to enter monitor");
       return;
   }
#endif
   rc = rdc_servo_set_port(port, period);
   if (rc < 0)
      throwControllerError(jnienv, "rdc_servo_set_port failed");
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorExit(jnienv, jcontroller);   
   if (rc < 0) {
       throwControllerError(jnienv, "failed to exit monitor");
   }
#endif
}


/*
 * Class:     Controller_Controller
 * Method:    encoder_enable_all
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_encoder_enable_all
    (JNIEnv *jnienv, jobject jcontroller, jint state)
{
    int rc;
#ifdef ROBOSKIFF_MONITORS
    rc = (*jnienv)->MonitorEnter(jnienv, jcontroller);
    if (rc < 0) {
        throwControllerError(jnienv, "failed to enter monitor");
        return;
    }
#endif
    rc = rdc_encoder_enable(state);
    if (rc < 0)
        throwControllerError(jnienv, "enable_encoders failed");
#ifdef ROBOSKIFF_MONITORS
    rc = (*jnienv)->MonitorExit(jnienv, jcontroller);   
    if (rc < 0) {
        throwControllerError(jnienv, "failed to exit monitor");
    }
#endif
}

/*
 * Class:     Controller_Controller
 * Method:    encoder_configure
 * Signature: (III)V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_encoder_configure
  (JNIEnv *jnienv, jobject jcontroller, jint port, jint low, jint high)
{
   int rc;
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorEnter(jnienv, jcontroller);
   if (rc < 0) {
       throwControllerError(jnienv, "failed to enter monitor");
       return;
   }
#endif
   rc = rdc_encoder_configure(port, low, high);
   if (rc < 0)
      throwControllerError(jnienv, "rdc_encoder_configure failed");
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorExit(jnienv, jcontroller);   
   if (rc < 0) {
       throwControllerError(jnienv, "failed to exit monitor");
   }
#endif
}

/*
 * Class:     Controller_Controller
 * Method:    encoder_get_counts
 * Signature: ([B)V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_encoder_get_counts
    (JNIEnv *jnienv, jobject jcontroller, jbyteArray jdata)
{
    jsize alen = (*jnienv)->GetArrayLength(jnienv, jdata);
   if ((*jnienv)->ExceptionOccurred(jnienv))
       return;
    if (alen >= ENC_NPORTS) {
        jbyte *data = (*jnienv)->GetByteArrayElements(jnienv, jdata, NULL);
        int rc;
#ifdef ROBOSKIFF_MONITORS
        rc = (*jnienv)->MonitorEnter(jnienv, jcontroller);
        if (rc < 0) {
            throwControllerError(jnienv, "failed to enter monitor");
            return;
        }
#endif
        rc = rdc_encoder_get_counts(data);
        (*jnienv)->ReleaseByteArrayElements(jnienv, jdata, data, JNI_ABORT);
        if (rc < 0)
            throwControllerError(jnienv, "encoder_get_counts failed");
#ifdef ROBOSKIFF_MONITORS
        rc = (*jnienv)->MonitorExit(jnienv, jcontroller);   
        if (rc < 0) {
            throwControllerError(jnienv, "failed to exit monitor");
        }
#endif
    } else {
        throwControllerError(jnienv, "encoder_get_counts: array too small");
    }
}

/*
 * Class:     Controller_Controller
 * Method:    encoder_reset_count
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_encoder_reset_count
    (JNIEnv *jnienv, jobject jcontroller, jint port)
{
    int rc;
#ifdef ROBOSKIFF_MONITORS
    rc = (*jnienv)->MonitorEnter(jnienv, jcontroller);
    if (rc < 0) {
        throwControllerError(jnienv, "failed to enter monitor");
        return;
    }
#endif
    rc = rdc_encoder_reset_count(port);
    if (rc < 0)
        throwControllerError(jnienv, "rdc_encoder_reset failed");
#ifdef ROBOSKIFF_MONITORS
    rc = (*jnienv)->MonitorExit(jnienv, jcontroller);   
    if (rc < 0) {
        throwControllerError(jnienv, "failed to exit monitor");
    }
#endif
}

/*
 * Class:     Controller_Controller
 * Method:    encoder_reset_counts
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_Controller_Controller_encoder_reset_counts
    (JNIEnv *jnienv, jobject jcontroller)
{
    int rc;
#ifdef ROBOSKIFF_MONITORS
    rc = (*jnienv)->MonitorEnter(jnienv, jcontroller);
    if (rc < 0) {
        throwControllerError(jnienv, "failed to enter monitor");
        return;
    }
#endif
    rc = rdc_encoder_reset_all_counts();
    if (rc < 0)
        throwControllerError(jnienv, "rdc_encoder_reset_all_counts failed");
#ifdef ROBOSKIFF_MONITORS
    rc = (*jnienv)->MonitorExit(jnienv, jcontroller);   
    if (rc < 0) {
        throwControllerError(jnienv, "failed to exit monitor");
    }
#endif
}

static JNINativeMethod nativeMethods[] = {
   { "usb_init", "()V", Java_Controller_Controller_usb_1init },
   { "aram_read", "(II[B)V", Java_Controller_Controller_aram_1read },
   { "aram_write", "(II[B)V", Java_Controller_Controller_aram_1write },
   { "eeprom_read", "(II[B)V", Java_Controller_Controller_eeprom_1read },
   { "eeprom_write", "(II[B)V", Java_Controller_Controller_eeprom_1write },
   { "get_eeprom_width", "()I", Java_Controller_Controller_get_1eeprom_1width },
   { "get_usb_chip_rev", "()I", Java_Controller_Controller_get_1usb_1chip_1rev },
   { "usb_renumerate", "()V", Java_Controller_Controller_usb_1renumerate },

   { "adc_get_values", "([B)I", Java_Controller_Controller_adc_get_values },
   { "servo_enable_port", "(II)V", Java_Controller_Controller_servo_enable_port },
   { "servo_set_port", "(II)V", Java_Controller_Controller_servo_set_port },
   { "encoder_enable_all", "(I)V", Java_Controller_Controller_encoder_enable_all },
   { "encoder_configure", "(III)V", Java_Controller_Controller_encoder_configure },
   { "encoder_get_counts", "([B)V", Java_Controller_Controller_encoder_get_counts },
   { "encoder_reset_count", "(I)V", Java_Controller_Controller_encoder_reset_count },
   { "encoder_reset_counts", "()V", Java_Controller_Controller_encoder_reset_counts },
};

JNIEXPORT void robotcontroller_Controller_registerNatives(JNIEnv *jnienv)
{
   jclass rdcClass = (*jnienv)->FindClass(jnienv, "robotcontroller.Controller");
   if ((*jnienv)->ExceptionOccurred(jnienv)) {
      fprintf(stderr, "exception occurred finding class robotcontroller.Controller\n");
      return;
   }
   (*jnienv)->RegisterNatives(jnienv, rdcClass, nativeMethods, sizeof(nativeMethods)/sizeof(JNINativeMethod));
   if ((*jnienv)->ExceptionOccurred(jnienv)) {
      fprintf(stderr, "exception occurred registering Controller natives\n");
      return;
   }
}
