/****************************************************************************/
/*  Copyright (c) Compaq Computer Corporation, 1999                         */
/*                                                                          */
/*  All Rights Reserved.  Unpublished rights reserved under the  copyright  */
/*  laws of the United States.                                              */
/*                                                                          */
/*  The software contained on this media is proprietary  to  and  embodies  */
/*  the   confidential   technology   of  Compaq Computer  Corporation.     */
/*  Possession, use, duplication or  dissemination  of  the  software  and  */
/*  media  is  authorized  only  pursuant  to a valid written license from  */
/*  Compaq Computer Corporation.                                            */
/****************************************************************************/

#include <jni.h>
#include "robotcontroller_LCD.h"
#include "rdcapi.h"

extern void throwControllerException(JNIEnv *jnienv, const char *msg);
extern void throwControllerError(JNIEnv *jnienv, const char *msg);

/*
 * Class:     robotcontroller_LCD
 * Method:    clear
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_robotcontroller_LCD_clear
  (JNIEnv *jnienv, jobject jlcd)
{
   int rc;
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorEnter(jnienv, jlcd);
   if (rc < 0) {
       throwControllerError(jnienv, "failed to enter monitor");
       return;
   }
#endif
   rc = rdc_lcd_clear();
   if (rc < 0)
      throwControllerError(jnienv, "rdc_lcd_clear failed");
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorExit(jnienv, jlcd);   
   if (rc < 0) {
       throwControllerError(jnienv, "failed to exit monitor");
   }
#endif
}


/*
 * Class:     robotcontroller_LCD
 * Method:    writeLine
 * Signature: (ILjava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_robotcontroller_LCD_writeLine
  (JNIEnv *jnienv, jobject jlcd, jint row, jstring jstr)
{
   const char *str = (*jnienv)->GetStringUTFChars(jnienv, jstr, NULL);
   int rc;
   char linebuf[17];
   int len;
   if ((*jnienv)->ExceptionOccurred(jnienv))
       return;
   len = (*jnienv)->GetStringUTFLength(jnienv, jstr);
   if ((*jnienv)->ExceptionOccurred(jnienv))
       return;
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorEnter(jnienv, jlcd);
   if (rc < 0) {
       throwControllerError(jnienv, "failed to enter monitor");
       return;
   }
#endif
   /* blank out the line */
   memset(linebuf, ' ', 17);
   if (len >= 16)
       strncpy(linebuf, str, 17); /* 17, to leave room for the terminating null character. */
   else
       strcpy(linebuf, str);
   rc = rdc_lcd_write_line(row, linebuf);
   if (rc < 0) {
      throwControllerError(jnienv, "rdc_lcd_write_line failed");
      return;
   }
   (*jnienv)->ReleaseStringUTFChars(jnienv, jstr, str);
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorExit(jnienv, jlcd);   
   if (rc < 0) {
       throwControllerError(jnienv, "failed to exit monitor");
   }
#endif
}


/*
 * Class:     robotcontroller_LCD
 * Method:    setCursor
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_robotcontroller_LCD_setCursor
  (JNIEnv *jnienv, jobject jlcd, jint row, jint column)
{
   int rc;
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorEnter(jnienv, jlcd);
   if (rc < 0) {
       throwControllerError(jnienv, "failed to enter monitor");
       return;
   }
#endif
   rc = rdc_lcd_set_pos(row, column);
   if (rc < 0)
      throwControllerError(jnienv, "rdc_lcd_set_pos failed");
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorExit(jnienv, jlcd);   
   if (rc < 0) {
       throwControllerError(jnienv, "failed to exit monitor");
   }
#endif
}

/*
 * Class:     robotcontroller_LCD
 * Method:    write_string
 * Signature: (Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_robotcontroller_LCD_writeString
  (JNIEnv *jnienv, jobject jlcd, jstring jstr)
{
   int len = (*jnienv)->GetStringUTFLength(jnienv, jstr);
   const char *str;
   int rc;
   if ((*jnienv)->ExceptionOccurred(jnienv))
       return;
   str = (*jnienv)->GetStringUTFChars(jnienv, jstr, NULL);
   if ((*jnienv)->ExceptionOccurred(jnienv))
       return;
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorEnter(jnienv, jlcd);
   if (rc < 0) {
       throwControllerError(jnienv, "failed to enter monitor");
       return;
   }
#endif
   rc = rdc_lcd_write_string(len, str);
   if (rc < 0) {
      throwControllerError(jnienv, "rdc_lcd_write_string failed");
      return;
   }
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorExit(jnienv, jlcd);   
   if (rc < 0) {
       throwControllerError(jnienv, "failed to exit monitor");
       return;
   }
#endif
   (*jnienv)->ReleaseStringUTFChars(jnienv, jstr, str);
}

static JNINativeMethod nativeMethods[] = {
   { "clear", "()V", Java_robotcontroller_LCD_clear },
   { "writeLine", "(ILjava/lang/String;)V", Java_robotcontroller_LCD_writeLine },
   { "setCursor", "(II)V", Java_robotcontroller_LCD_setCursor },
   { "writeString", "(Ljava/lang/String;)V", Java_robotcontroller_LCD_writeString },
};

JNIEXPORT void robotcontroller_LCD_registerNatives(JNIEnv *jnienv)
{
   jclass lcdClass = (*jnienv)->FindClass(jnienv, "robotcontroller.LCD");
   if ((*jnienv)->ExceptionOccurred(jnienv)) {
      fprintf(stderr, "error occurred finding class robotcontroller.LCD\n");
      return;
   }
   (*jnienv)->RegisterNatives(jnienv, lcdClass, nativeMethods, sizeof(nativeMethods)/sizeof(JNINativeMethod));
   if ((*jnienv)->ExceptionOccurred(jnienv)) {
      fprintf(stderr, "error occurred registering LCD natives\n");
      return;
   }
}
