/****************************************************************************/
/*  Copyright (c) Compaq Computer Corporation, 1999                         */
/*                                                                          */
/*  All Rights Reserved.  Unpublished rights reserved under the  copyright  */
/*  laws of the United States.                                              */
/*                                                                          */
/*  The software contained on this media is proprietary  to  and  embodies  */
/*  the   confidential   technology   of  Compaq Computer  Corporation.     */
/*  Possession, use, duplication or  dissemination  of  the  software  and  */
/*  media  is  authorized  only  pursuant  to a valid written license from  */
/*  Compaq Computer Corporation.                                            */
/****************************************************************************/


#include <jni.h>
#include "robotcontroller_Motor.h"
#include "rdcapi.h"

/*
 * Class:     robotcontroller_Motor
 * Method:    set_speed
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_robotcontroller_Motor_setSpeed
  (JNIEnv *jnienv, jclass motorclass, jint port, jint speed)
{
   int rc;
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorEnter(jnienv, motorclass);
   if (rc < 0) {
       throwControllerError(jnienv, "failed to enter monitor");
       return;
   }
#endif
   rc = rdc_motor_set_speed(port, speed);
   if (rc < 0)
      throwControllerException(jnienv, "rdc_motor_set_speed failed");
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorExit(jnienv, motorclass);   
   if (rc < 0) {
       throwControllerError(jnienv, "failed to exit monitor");
   }
#endif
}

/*
 * Class:     robotcontroller_Motor
 * Method:    stop_all
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_robotcontroller_Motor_stopAll
  (JNIEnv *jnienv, jclass motorclass)
{
   int rc;
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorEnter(jnienv, motorclass);
   if (rc < 0) {
       throwControllerError(jnienv, "failed to enter monitor");
       return;
   }
#endif
   rc = rdc_motor_stop_all_motors();
   if (rc < 0)
      throwControllerException(jnienv, "rdc_motor_stop_all_motors failed");
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorExit(jnienv, motorclass);   
   if (rc < 0) {
       throwControllerError(jnienv, "failed to exit monitor");
   }
#endif
}

/*
 * Class:     robotcontroller_Motor
 * Method:    set_motor_pair
 * Signature: (III)V
 */
JNIEXPORT void JNICALL Java_robotcontroller_Motor_setMotorPair
  (JNIEnv *jnienv, jclass motorclass, jint motor_num, jint second_motor_num, jint speed)
{
   int rc;
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorEnter(jnienv, motorclass);
   if (rc < 0) {
       throwControllerError(jnienv, "failed to enter monitor");
       return;
   }
#endif
   rc = rdc_motor_set_motor_pair(motor_num, second_motor_num, speed);
   if (rc < 0)
      throwControllerException(jnienv, "rdc_motor_set_motor_pair failed");
#ifdef ROBOSKIFF_MONITORS
   rc = (*jnienv)->MonitorExit(jnienv, motorclass);   
   if (rc < 0) {
       throwControllerError(jnienv, "failed to exit monitor");
   }
#endif
}

static JNINativeMethod nativeMethods[] = {
   { "setSpeed", "(II)V", Java_robotcontroller_Motor_setSpeed },
   { "stopAll", "()V", Java_robotcontroller_Motor_stopAll },
   { "setMotorPair", "(III)V", Java_robotcontroller_Motor_setMotorPair },
};

JNIEXPORT void robotcontroller_Motor_registerNatives(JNIEnv *jnienv)
{
   jclass motorClass = (*jnienv)->FindClass(jnienv, "robotcontroller.Motor");
   if ((*jnienv)->ExceptionOccurred(jnienv)) {
      fprintf(stderr, "exception occurred finding class robotcontroller.Motor\n");
      return;
   }
   (*jnienv)->RegisterNatives(jnienv, motorClass, nativeMethods, sizeof(nativeMethods)/sizeof(JNINativeMethod));
   if ((*jnienv)->ExceptionOccurred(jnienv)) {
      fprintf(stderr, "exception occurred registering Motor natives\n");
      return;
   }
}
